@@ -2,7 +2,7 @@
  * cvd.c : CVD Subtitle decoder
  *****************************************************************************
  * Copyright (C) 2003, 2004 the VideoLAN team
- * $Id: cvdsub.c 13905 2006-01-12 23:10:04Z dionoea $
+ * $Id: 48da421c6c9cd6d0c14b8a660380407d5b747653 $
  *
  * Authors: Rocky Bernstein
  *          Gildas Bazin <gbazin@videolan.org>
@@ -27,9 +27,14 @@
 /*****************************************************************************
  * Preamble
  *****************************************************************************/
-#include <vlc/vlc.h>
-#include <vlc/vout.h>
-#include <vlc/decoder.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
+#include <vlc_common.h>
+#include <vlc_plugin.h>
+#include <vlc_vout.h>
+#include <vlc_codec.h>
 
 #include "vlc_bits.h"
 
@@ -43,12 +48,12 @@ static int  PacketizerOpen( vlc_object_t * );
 static void DecoderClose  ( vlc_object_t * );
 
 vlc_module_begin();
-    set_description( _("CVD subtitle decoder") );
+    set_description( N_("CVD subtitle decoder") );
     set_capability( "decoder", 50 );
     set_callbacks( DecoderOpen, DecoderClose );
 
     add_submodule();
-    set_description( _("Chaoji VCD subtitle packetizer") );
+    set_description( N_("Chaoji VCD subtitle packetizer") );
     set_capability( "packetizer", 50 );
     set_callbacks( PacketizerOpen, DecoderClose );
 vlc_module_end();
@@ -76,16 +81,16 @@ struct decoder_sys_t
 
   block_t  *p_spu;   /* Bytes of the packet. */
 
-  int     i_spu_size;     /* goal for subtitle_data_pos while gathering,
+  size_t   i_spu_size;     /* goal for subtitle_data_pos while gathering,
                              size of used subtitle_data later */
 
   uint16_t i_image_offset;      /* offset from subtitle_data to compressed
                                    image data */
-  int i_image_length;           /* size of the compressed image data */
-  int first_field_offset;       /* offset of even raster lines */
-  int second_field_offset;      /* offset of odd raster lines */
-  int metadata_offset;          /* offset to data describing the image */
-  int metadata_length;          /* length of metadata */
+  size_t i_image_length;           /* size of the compressed image data */
+  size_t first_field_offset;       /* offset of even raster lines */
+  size_t second_field_offset;      /* offset of odd raster lines */
+  size_t metadata_offset;          /* offset to data describing the image */
+  size_t metadata_length;          /* length of metadata */
 
   mtime_t i_duration;   /* how long to display the image, 0 stands
                            for "until next subtitle" */
@@ -112,8 +117,10 @@ static int DecoderOpen( vlc_object_t *p_this )
     }
 
     p_dec->p_sys = p_sys = malloc( sizeof( decoder_sys_t ) );
+    if( !p_sys )
+        return VLC_ENOMEM;
 
-    p_sys->b_packetizer  = VLC_FALSE;
+    p_sys->b_packetizer  = false;
 
     p_sys->i_state = SUBTITLE_BLOCK_EMPTY;
     p_sys->p_spu   = NULL;
@@ -135,7 +142,7 @@ static int PacketizerOpen( vlc_object_t *p_this )
 
     if( DecoderOpen( p_this ) != VLC_SUCCESS ) return VLC_EGENERIC;
 
-    p_dec->p_sys->b_packetizer = VLC_TRUE;
+    p_dec->p_sys->b_packetizer = true;
 
     return VLC_SUCCESS;
 }
@@ -213,7 +220,7 @@ static block_t *Reassemble( decoder_t *p_dec, block_t *p_block )
 
     if( p_block->i_buffer < SPU_HEADER_LEN )
     {
-        msg_Dbg( p_dec, "invalid packet header (size %d < %d)" ,
+        msg_Dbg( p_dec, "invalid packet header (size %zu < %u)" ,
                  p_block->i_buffer, SPU_HEADER_LEN );
         block_Release( p_block );
         return NULL;
@@ -246,11 +253,11 @@ static block_t *Reassemble( decoder_t *p_dec, block_t *p_block )
 
         if( p_spu->i_buffer != p_sys->i_spu_size )
         {
-            msg_Warn( p_dec, "SPU packets size=%d should be %d",
+            msg_Warn( p_dec, "SPU packets size=%zu should be %zu",
                       p_spu->i_buffer, p_sys->i_spu_size );
         }
 
-        msg_Dbg( p_dec, "subtitle packet complete, size=%d", p_spu->i_buffer);
+        msg_Dbg( p_dec, "subtitle packet complete, size=%zuu", p_spu->i_buffer);
 
         ParseMetaInfo( p_dec, p_spu );
 
@@ -272,14 +279,14 @@ static block_t *Reassemble( decoder_t *p_dec, block_t *p_block )
   except the submux sample code and a couple of samples of dubious
   origin. Thus, this is the result of reading some code whose
   correctness is not known and some experimentation.
-  
+
   CVD subtitles are different in several ways from SVCD OGT subtitles.
   Image comes first and metadata is at the end.  So that the metadata
   can be found easily, the subtitle packet starts with two bytes
   (everything is big-endian again) that give the total size of the
   subtitle data and the offset to the metadata - i.e. size of the
   image data plus the four bytes at the beginning.
- 
+
   Image data comes interlaced is run-length encoded.  Each field is a
   four-bit nibble. Each nibble contains a two-bit repeat count and a
   two-bit color number so that up to three pixels can be described in
@@ -306,16 +313,15 @@ static void ParseHeader( decoder_t *p_dec, block_t *p_block )
 
     p_sys->i_image_offset = 4;
     p_sys->i_image_length = p_sys->metadata_offset - p_sys->i_image_offset;
-  
+
 #ifdef DEBUG_CVDSUB
-    msg_Dbg( p_dec, "total size: %d  image size: %d",
+    msg_Dbg( p_dec, "total size: %zu  image size: %zu",
              p_sys->i_spu_size, p_sys->i_image_length );
 #endif
-
 }
 
-/* 
-  We parse the metadata information here. 
+/*
+  We parse the metadata information here.
 
   Although metadata information does not have to come in a fixed field
   order, every metadata field consists of a tag byte followed by
@@ -333,7 +339,7 @@ static void ParseMetaInfo( decoder_t *p_dec, block_t *p_spu  )
     decoder_sys_t *p_sys = p_dec->p_sys;
     uint8_t       *p     = p_spu->p_buffer + p_sys->metadata_offset;
     uint8_t       *p_end = p + p_sys->metadata_length;
-  
+
     for( ; p < p_end; p += 4 )
     {
         switch( p[0] )
@@ -347,14 +353,14 @@ static void ParseMetaInfo( decoder_t *p_dec, block_t *p_spu  )
 #endif
             p_sys->i_duration *= 100 / 9;
             break;
-      
+
         case 0x0c: /* unknown */
 #ifdef DEBUG_CVDSUB
             msg_Dbg( p_dec, "subtitle command unknown 0x%0x 0x%0x 0x%0x 0x%0x",
                      (int)p[0], (int)p[1], (int)p[2], (int)p[3] );
 #endif
             break;
-      
+
         case 0x17: /* coordinates of subtitle upper left x, y position */
             ExtractXY(p_sys->i_x_start, p_sys->i_y_start);
 
@@ -363,7 +369,7 @@ static void ParseMetaInfo( decoder_t *p_dec, block_t *p_spu  )
                      p_sys->i_x_start, p_sys->i_y_start );
 #endif
             break;
-      
+
         case 0x1f: /* coordinates of subtitle bottom right x, y position */
         {
             int lastx;
@@ -378,11 +384,11 @@ static void ParseMetaInfo( decoder_t *p_dec, block_t *p_spu  )
 #endif
             break;
         }
-      
+
         case 0x24:
         case 0x25:
         case 0x26:
-        case 0x27: 
+        case 0x27:
         {
             uint8_t v = p[0] - 0x24;
 
@@ -431,7 +437,7 @@ static void ParseMetaInfo( decoder_t *p_dec, block_t *p_spu  )
                      (int)p_sys->p_palette[2][3], (int)p_sys->p_palette[3][3]);
 #endif
             break;
-      
+
         case 0x3f:
             /* transparency for highlight palette */
             p_sys->p_palette_highlight[0][3] = (p[2] & 0x0f) << 4;
@@ -455,7 +461,8 @@ static void ParseMetaInfo( decoder_t *p_dec, block_t *p_spu  )
             p_sys->first_field_offset =
                 (p[2] << 8) + p[3] - p_sys->i_image_offset;
 #ifdef DEBUG_CVDSUB
-            msg_Dbg( p_dec, "1st_field_offset %d", p_sys->first_field_offset );
+            msg_Dbg( p_dec, "1st_field_offset %zu",
+                     p_sys->first_field_offset );
 #endif
             break;
 
@@ -465,13 +472,14 @@ static void ParseMetaInfo( decoder_t *p_dec, block_t *p_spu  )
             p_sys->second_field_offset =
                 (p[2] << 8) + p[3] - p_sys->i_image_offset;
 #ifdef DEBUG_CVDSUB
-            msg_Dbg( p_dec, "2nd_field_offset %d", p_sys->second_field_offset);
+            msg_Dbg( p_dec, "2nd_field_offset %zu",
+                     p_sys->second_field_offset);
 #endif
             break;
 
         default:
 #ifdef DEBUG_CVDSUB
-            msg_Warn( p_dec, "unknown sequence in control header " 
+            msg_Warn( p_dec, "unknown sequence in control header "
                       "0x%0x 0x%0x 0x%0x 0x%0x", p[0], p[1], p[2], p[3]);
 #endif
         }
@@ -496,12 +504,14 @@ static subpicture_t *DecodePacket( decoder_t *p_dec, block_t *p_data )
     p_spu = p_dec->pf_spu_buffer_new( p_dec );
     if( !p_spu ) return NULL;
 
+    p_spu->b_pausable = true;
+
     p_spu->i_x = p_sys->i_x_start;
     p_spu->i_x = p_spu->i_x * 3 / 4; /* FIXME: use aspect ratio for x? */
     p_spu->i_y = p_sys->i_y_start;
     p_spu->i_start = p_data->i_pts;
     p_spu->i_stop  = p_data->i_pts + p_sys->i_duration;
-    p_spu->b_ephemer = VLC_TRUE;
+    p_spu->b_ephemer = true;
 
     /* Create new SPU region */
     memset( &fmt, 0, sizeof(video_format_t) );
@@ -538,7 +548,7 @@ static subpicture_t *DecodePacket( decoder_t *p_dec, block_t *p_data )
 /*****************************************************************************
  * ParseImage: parse and render the image part of the subtitle
  *****************************************************************************
- This part parses the subtitle graphical data and renders it. 
+ This part parses the subtitle graphical data and renders it.
 
  Image data comes interlaced and is run-length encoded (RLE). Each
  field is a four-bit nibbles that is further subdivided in a two-bit
@@ -554,7 +564,7 @@ static subpicture_t *DecodePacket( decoder_t *p_dec, block_t *p_data )
  may be untested.
 
  However we'll transform this so that that the RLE is expanded and
- interlacing will also be removed. On output each pixel entry will by 
+ interlacing will also be removed. On output each pixel entry will by
  a 4-bit alpha (filling 8 bits), and 8-bit y, u, and v entry.
 
  *****************************************************************************/
